(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit AboutDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, AboutDlgType;

type
  TAboutDlg = class(TComponent)
  private
    { Dclarations prives }
    FComments : string;
    FBitmap   : TBitmap;
    FAuthor   : TStrings;
    procedure SetAuthor(Value: TStrings);
    procedure SetBitmap(Value: TBitmap);
  protected
    { Dclarations protges }
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function Execute: Boolean;
    function ExecuteSimpleAbout: Boolean;
    function ExecuteAbout2 : boolean;
  published
    { Dclarations publies }
    property Comments : string   read FComments write FComments;
    property Bitmap   : TBitmap  read FBitmap   write SetBitmap;
    property Author   : TStrings read FAuthor   write SetAuthor;
  end;

implementation

function TAboutDlg.Execute: Boolean;
begin
  result := ExecuteSimpleAbout;
end;

function TAboutDlg.ExecuteSimpleAbout: Boolean;
var
  Hndl : Integer;
begin
  if not FBitmap.Empty
    then Hndl := FBitmap.Handle
    else Hndl := -1;
  OpenChildAbout(Application.MainForm.Handle, PChar(Application.ExeName), PChar (FComments), Hndl);
  result := true;
end;

function TAboutDlg.ExecuteAbout2: boolean;
var Hndl  : Integer;
    R1, R2: PResourceParameter;
begin
  if not FBitmap.Empty then
    Hndl := FBitmap.Handle
  else
    Hndl := -1;
  R1 := nil; R2 := nil;
  if(FindResource(HInstance, 'AUTHORS', 'ABOUT') <> 0) then begin
    GetMem(R1, sizeof(R1^));
    R1.Instance := HInstance;
    R1.ResType  := 'ABOUT';
    R1.ResName  := 'AUTHORS';
  end;
  if(FindResource(HInstance, 'LICENCE', 'ABOUT') <> 0) then begin
    GetMem(R2, sizeof(R2^));
    R2.Instance := HInstance;
    R2.ResType  := 'ABOUT';
    R2.ResName  := 'LICENCE';
  end;
  try
    OpenAbout2(Application.MainForm.Handle, PChar(Application.ExeName), Hndl, R1, R2);
  finally
    FreeMem(R1);
    FreeMem(R2);
  end;

  result := true;
end;

procedure TAboutDlg.SetAuthor(Value: TStrings);
begin
  FAuthor.Assign(Value);
end;

procedure TAboutDlg.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
end;

constructor TAboutDlg.Create(AOwner: TComponent);
begin
  inherited Create (AOwner);
  FAuthor := TStringList.Create;
  FBitmap := TBitmap.Create;
end;

destructor TAboutDlg.Destroy;
begin
  FBitmap.Free;
  FAuthor.Free;
  inherited Destroy;
end;

end.
