unit BrowseDr;

interface

uses
  Windows, SysUtils, ShlObj, Controls, Classes, Forms, Dialogs, Messages, ActiveX;

const
  CSIDL_DESKTOPEXPANDED  = $FEFE; { This is undocumented, but should work for a long time}

type
  TRootID = (idDesktop, idInternet, idPrograms, idControlPanel, idPrinters, idPersonal,
    idFavorites, idStartup, idRecent, idSendTo, idRecycleBin, idStartMenu,
    idDesktopDirectory, idDrives, idNetwork, idNetHood, idFonts, idTemplates,
    idCommonStartmenu, idCommonPrograms, idCommonStartup, idCommonDesktopDirectory,
    idAppData, idPrintHood, idAltStartup, idCommonAltStartup, idCommonFavorites,
    idInternetCache, idCookies, idHistory, idDesktopExpanded);

  TBrowseFlag = (bfDirectoriesOnly, bfDomainOnly, bfAncestors, bfComputers, bfPrinters,
    bfIncludeFiles, bfIncludeURLs, bfNewDialogStyle, bfEdit);
  TBrowseFlags = set of TBrowseFlag;

  TBDSelChangedEvent = procedure(Sender: TObject; const NewSel: string) of object;

type
  TBrowseDirectoryDlg = class(TComponent)
  private
    FDlgWnd: HWND;

    FCaption: string;
    FParent: TWinControl;
    FShowSelectionInStatus: boolean;
    FFitStatusText: boolean;

    FTitle: string;
    FRoot: TRootID;
    FOptions: TBrowseFlags;
    FSelection: string;
    FCenter: boolean;
    FStatusText: string;
    FEnableOKButton: boolean;
    FImageIndex: integer;
    FSelChanged: TBDSelChangedEvent;
    FOnCreate: TNotifyEvent;
  protected
    function FittedStatusText: string;

    procedure DoInitialized(Wnd: HWND); virtual;
    procedure DoSelChanged(Wnd: HWND; Item: PItemIDList); virtual;

    procedure SetFitStatusText(Val: boolean);
    procedure SetStatusText(const Val: string);
    procedure SetSelection(const Val: string);
    procedure SetEnableOKButton(Val: boolean);
    function GetCaption: string;
    procedure SetCaption(const Val: string);
    procedure SetParent(AParent: TWinControl);
  public
    constructor Create(AOwner: TComponent); override;

    function Execute: boolean; virtual;

    property Parent: TWinControl read FParent write SetParent;
  published
    property Selection: string read FSelection write SetSelection;
    property Title: string read FTitle write FTitle;
    property Root: TRootID read FRoot write FRoot default idDesktop;
    property Options: TBrowseFlags read FOptions write FOptions default [];
    property Center: boolean read FCenter write FCenter default TRUE;

    property StatusText: string read FStatusText write SetStatusText;
    property FitStatusText: boolean read FFitStatusText write SetFitStatusText default TRUE;
    property EnableOKButton: boolean read FEnableOKButton write SetEnableOKButton default TRUE;

    property ImageIndex: integer read FImageIndex;
    property Caption: string read GetCaption write SetCaption;
    property ShowSelectionInStatus: boolean read FShowSelectionInStatus write FShowSelectionInStatus;

    property OnSelChanged: TBDSelChangedEvent read FSelChanged write FSelChanged;
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
  end;

implementation

function ConvertRoot(Root: TRootID): integer;
const RootValues : array[TRootID] of integer
                 = (CSIDL_DESKTOP, CSIDL_INTERNET, CSIDL_PROGRAMS, CSIDL_CONTROLS,
                    CSIDL_PRINTERS, CSIDL_PERSONAL, CSIDL_FAVORITES, CSIDL_STARTUP,
                    CSIDL_RECENT, CSIDL_SENDTO, CSIDL_BITBUCKET, CSIDL_STARTMENU,
                    CSIDL_DESKTOPDIRECTORY, CSIDL_DRIVES, CSIDL_NETWORK, CSIDL_NETHOOD,
                    CSIDL_FONTS, CSIDL_TEMPLATES, CSIDL_COMMON_STARTMENU,
                    CSIDL_COMMON_PROGRAMS, CSIDL_COMMON_STARTUP,
                    CSIDL_COMMON_DESKTOPDIRECTORY, CSIDL_APPDATA, CSIDL_PRINTHOOD,
                    CSIDL_ALTSTARTUP, CSIDL_COMMON_ALTSTARTUP, CSIDL_COMMON_FAVORITES,
                    CSIDL_INTERNET_CACHE, CSIDL_COOKIES, CSIDL_HISTORY,
                    CSIDL_DESKTOPEXPANDED);
begin
  Result := RootValues[Root];
end;

function ConvertFlags(Flags: TBrowseFlags): UINT;
const FlagValues : array[TBrowseFlag] of UINT
                 = (BIF_RETURNONLYFSDIRS, BIF_DONTGOBELOWDOMAIN, BIF_RETURNFSANCESTORS,
                    BIF_BROWSEFORCOMPUTER, BIF_BROWSEFORPRINTER, BIF_BROWSEINCLUDEFILES,
                    BIF_BROWSEINCLUDEURLS, BIF_NEWDIALOGSTYLE, BIF_EDITBOX);
var Opt: TBrowseFlag;
begin
  Result := 0;
  { Loop through all possible values }
  for Opt := Low(TBrowseFlag) to High(TBrowseFlag) do
    if(Opt in Flags)then
      Result := Result or FlagValues[Opt];
end;

function GetTextWidth(DC: HDC; const Text: String): Integer;
var Extent: TSize;
begin
  if(GetTextExtentPoint(DC, PChar(Text), Length(Text), Extent))then
    Result := Extent.cX
  else
    Result := 0;
end;

function MinimizeName(Wnd: HWND; const Filename: string): string;

  procedure CutFirstDirectory(var S: string);
  var
    Root: Boolean;
    P: Integer;
  begin
    if S = '\' then
      S := ''
    else begin
      if S[1] = '\' then begin
        Root := True;
        Delete(S, 1, 1);
      end else
        Root := False;
      if S[1] = '.' then
        Delete(S, 1, 4);
      P := Pos('\',S);
      if P <> 0 then begin
        Delete(S, 1, P);
        S := '...\' + S;
      end else
        S := '';
      if Root then

        S := '\' + S;
    end;
  end;

var
  Drive: string;
  Dir: string;
  Name: string;
  R: TRect;
  DC: HDC;
  MaxLen: integer;
  OldFont, Font: HFONT;
begin
  Result := FileName;
  if Wnd = 0 then exit;
  DC := GetDC(Wnd);
  if DC = 0 then exit;
  Font := SendMessage(Wnd, WM_GETFONT, 0, 0);
  OldFont := SelectObject(DC, Font);
  try
    GetWindowRect(Wnd, R);
    MaxLen := R.Right - R.Left;

    Dir := ExtractFilePath(Result);
    Name := ExtractFileName(Result);

    if (Length(Dir) >= 2) and (Dir[2] = ':') then begin
      Drive := Copy(Dir, 1, 2);
      Delete(Dir, 1, 2);
    end else
      Drive := '';
    while ((Dir <> '') or (Drive <> '')) and (GetTextWidth(DC, Result) > MaxLen) do begin
      if Dir = '\...\' then begin
        Drive := '';
        Dir := '...\';
      end else if Dir = '' then
        Drive := ''
      else
        CutFirstDirectory(Dir);
      Result := Drive + Dir + Name;
    end;
  finally

    SelectObject(DC, OldFont);
    ReleaseDC(Wnd, DC);
  end;
end;

function MinimizeString(Wnd: HWND; const Text: string): string;
var
  R: TRect;
  DC: HDC;
  MaxLen: integer;
  OldFont, Font: HFONT;
  TempStr: string;
begin
  Result := Text;
  TempStr := Text;
  if Wnd = 0 then exit;
  DC := GetDC(Wnd);
  if DC = 0 then exit;
  Font := SendMessage(Wnd, WM_GETFONT, 0, 0);
  OldFont := SelectObject(DC, Font);
  try
    GetWindowRect(Wnd, R);
    MaxLen := R.Right - R.Left;

    while (TempStr <> '') and (GetTextWidth(DC, Result) > MaxLen) do begin
      SetLength(TempStr, Length(TempStr)-1);
      Result := TempStr + '...';
    end;
  finally
    SelectObject(DC, OldFont);
    ReleaseDC(Wnd, DC);
  end;
end;

function BrowseCallbackProc(Wnd: HWnd; Msg: UINT; lParam: LPARAM; lData: LPARAM): integer; stdcall;
begin
  Result := 0;
  case Msg of
    BFFM_INITIALIZED: begin
      if(lData <> 0)then
        TBrowseDirectoryDlg(lData).DoInitialized(Wnd);
    end;
    BFFM_SELCHANGED: begin
      if(lData <> 0)then
        TBrowseDirectoryDlg(lData).DoSelChanged(Wnd, PItemIDList(lParam));
    end;
  end;
end;


function BrowseDirectory(var Dest: string; var ImgIdx: integer;
                         const AParent: TWinControl; const Title: string; Root: TRootID;
                         Flags: TBrowseFlags; WantStatusText: boolean;
                         Callback: TFNBFFCallBack; Data: Longint): boolean;
var ShellMalloc: IMALLOC;
    shBuff: PChar;
    BrowseInfo: TBrowseInfo;
    idRoot, idBrowse: PItemIDList;
    WndHandle: HWND;
begin
  Result := false;

  Dest := ''; 
  SetLength(Dest, MAX_PATH);  
  if(Assigned(AParent)) then
    WndHandle := AParent.Handle
  else
    WndHandle := 0;

  if(SHGetMalloc(ShellMalloc) = NOERROR) then try
    shBuff := PChar(ShellMalloc.Alloc(MAX_PATH));
    if(assigned(shBuff)) then try
      SHGetSpecialFolderLocation(WndHandle, ConvertRoot(Root), idRoot);

      try
        with BrowseInfo do begin
          hwndOwner := WndHandle;
          pidlRoot := idRoot;
          pszDisplayName := shBuff;
          lpszTitle := PChar(Title);
          ulFlags := ConvertFlags(Flags);
          if(WantStatusText)then
            ulFlags := ulFlags or BIF_STATUSTEXT;
          lpfn := Callback;
          lParam := Data;
        end;
        idBrowse := SHBrowseForFolder(BrowseInfo);

        if(assigned(idBrowse))then try
          SHGetPathFromIDList(idBrowse, shBuff);
          Dest := shBuff;
          ImgIdx := BrowseInfo.iImage;
          Result := true;
        finally
          ShellMalloc.Free(idBrowse);
        end;
      finally
        ShellMalloc.Free(idRoot);
      end;
    finally
      ShellMalloc.Free(shBuff);
    end;
  finally
    ShellMalloc._Release; // Clean-up.
  end;
end;

constructor TBrowseDirectoryDlg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDlgWnd         := 0;
  FFitStatusText  := true;
  FEnableOKButton := true;

  FTitle          := '';
  FRoot           := idDesktop;
  FOptions        := [];
  FSelection      := '';
  FCenter         := TRUE;
  FSelChanged     := NIL;
  FStatusText     := '';
  FImageIndex     := -1;
  FCaption        := '';

  if(Assigned(AOwner))then
    if(AOwner is TWinControl) then
      FParent := TWinControl(Owner)
    else if((Assigned(Application)) and (Assigned(Application.MainForm))) then
      FParent := Application.MainForm;
end;

function TBrowseDirectoryDlg.Execute: boolean;
var S: string;
    AParent: TWinControl;
begin
  AParent := NIL;
  if(not (csDesigning in ComponentState))then
    if(Assigned(FParent)) then
      AParent := FParent
    else begin
      if(Assigned(Owner)) then
        if(Owner is TWinControl) then
          AParent := TWinControl(Owner)
        else
          if(Assigned(Application) and Assigned(Application.MainForm)) then
            AParent := Application.MainForm;
    end;

  Result := BrowseDirectory(S, FImageIndex, AParent, FTitle, FRoot, FOptions,
                            FStatusText <> '', BrowseCallbackProc, LongInt(Self));

  FDlgWnd := 0;

  if Result then
    Selection := S
  else
    Selection := ''
end;

function FormatSelection(const APath: string): string;
begin
  Result := APath;

  if(Result <> '') then begin
    if((Length(Result) < 4) and (Result[2] = ':')) then begin
      if(Length(Result) = 2) then
        Result := Result + '\'
    end else
      if(Result[Length(Result)] = '\') then
        SetLength(Result, Length(Result)-1);
  end;
end;

procedure TBrowseDirectoryDlg.DoInitialized(Wnd: HWND);
var
  Rect: TRect;
begin
  FDlgWnd := Wnd;
  if(FCenter) then begin
    GetWindowRect(Wnd, Rect);
    SetWindowPos(Wnd, 0,
      (GetSystemMetrics(SM_CXSCREEN) - Rect.Right + Rect.Left) div 2,
      (GetSystemMetrics(SM_CYSCREEN) - Rect.Bottom + Rect.Top) div 2,
      0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
  end;

  SendMessage(FDlgWnd, BFFM_ENABLEOK, 0, LPARAM(FEnableOKButton));
  if(FStatusText <> '') then
    SendMessage(Wnd, BFFM_SETSTATUSTEXT, 0, LPARAM(FittedStatusText));
  if(FSelection <> '') then
    SendMessage(FDlgWnd, BFFM_SETSELECTION, 1, LPARAM(FormatSelection(FSelection)));

  if(FCaption <> '') then
    SendMessage(FDlgWnd, WM_SETTEXT, 0, LPARAM(FCaption));
  if(Assigned(FOnCreate)) then
    FOnCreate(Self);
end;

procedure TBrowseDirectoryDlg.DoSelChanged(Wnd: HWND; Item: PItemIDList);
var
  Name: string;
begin
  if(FShowSelectionInStatus or Assigned(FSelChanged)) then begin
    Name := '';
    SetLength(Name, MAX_PATH);
    SHGetPathFromIDList(Item, PChar(Name));
    SetLength(Name, StrLen(PChar(Name)));
    if FShowSelectionInStatus then
      StatusText := Name;

    if assigned(FSelChanged) then
      FSelChanged(Self, Name);
  end;
end;

procedure TBrowseDirectoryDlg.SetFitStatusText(Val: boolean);
begin
  if(FFitStatusText = Val) then exit;
  FFitStatusText := Val;

  if(FDlgWnd <> 0) then
    SendMessage(FDlgWnd, BFFM_SETSTATUSTEXT, 0, LPARAM(FittedStatusText));
end;

procedure TBrowseDirectoryDlg.SetStatusText(const Val: string);
begin
  if(FStatusText = Val) then exit;

  FStatusText := Val;
  if(FDlgWnd <> 0) then
    SendMessage(FDlgWnd, BFFM_SETSTATUSTEXT, 0, LPARAM(FittedStatusText));
end;

procedure TBrowseDirectoryDlg.SetSelection(const Val: string);
begin
  if FSelection = Val then exit;
  FSelection := Val;

  if (FSelection <> '') and (FSelection[Length(FSelection)] <> '\') and
     DirectoryExists(FSelection) then
    FSelection := FSelection + '\';
  if(FDlgWnd <> 0) then begin

    if(FSelection <> '') then
      SendMessage(FDlgWnd, BFFM_SETSELECTION, 1, LPARAM(FormatSelection(FSelection)));
  end;
end;

procedure TBrowseDirectoryDlg.SetEnableOKButton(Val: boolean);
begin
  if(FEnableOKButton = Val) then exit;
  FEnableOKButton := Val;
  if(FDlgWnd <> 0) then
    SendMessage(FDlgWnd, BFFM_ENABLEOK, 0, LPARAM(FEnableOKButton));
end;

function TBrowseDirectoryDlg.GetCaption: string;
var Temp: array[0..255] of char;
begin
  if(FDlgWnd <> 0) then begin
    SendMessage(FDlgWnd, WM_GETTEXT, SizeOf(Temp), LPARAM(@Temp));
    Result := string(Temp);
  end else
    Result := FCaption;
end;

procedure TBrowseDirectoryDlg.SetCaption(const Val: string);
begin
  FCaption := Val;
  if(FDlgWnd <> 0) then
    SendMessage(FDlgWnd, WM_SETTEXT, 0, LPARAM(FCaption));
end;

procedure TBrowseDirectoryDlg.SetParent(AParent: TWinControl);
begin
  FParent := AParent;
end;

function EnumChildWndProc(Child: HWND; Data: LParam): BOOL; stdcall;
const STATUS_TEXT_WINDOW_ID = 14147;
type PHWND = ^HWND;
begin
  if(GetWindowLong(Child, GWL_ID) = STATUS_TEXT_WINDOW_ID) then begin
    PHWND(Data)^ := Child;
    Result := FALSE;
  end else
    Result := TRUE;
end;

function TBrowseDirectoryDlg.FittedStatusText: string;
var ChildWnd: HWND;
begin
  Result := FStatusText;
  if(FFitStatusText) then begin

    ChildWnd := 0;
    if(FDlgWnd <> 0) then
      EnumChildWindows(FDlgWnd, @EnumChildWndProc, LPARAM(@ChildWnd));
    if((ChildWnd <> 0) and (FStatusText <> '')) then
      if(DirectoryExists(FStatusText)) then
        Result := MinimizeName(ChildWnd, FStatusText)
      else
        Result := MinimizeString(ChildWnd, FStatusText);
  end;
end;

end.