(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit DebugBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TPositions = (poTopleft, poBottomLeft, poTopRight, poBottomRight);

  TDebugBox = class(TComponent)
  private
    { Dclarations prives }
    DebugForm : TForm;
    DebugList : TListBox;
    FPosition : TPositions;
    FStamp,
    FVisible  : Boolean;
    FIndent,
    FWidth,
    FHeight   : Integer;
    FCaption  : String;
    procedure SetPosition(Value: TPositions);
    procedure SetVisible (Value: Boolean);
    procedure SetWidth   (Value: Integer);
    procedure SetHeight  (Value: integer);
    procedure SetCaption (Value: String);
  protected
    { Dclarations protges }
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    procedure Clear;
    procedure Add(Value: String);
  published
    { Dclarations publies }
    property Caption   : String     read FCaption  write SetCaption;
    property Position  : TPositions read FPosition write SetPosition default poTopRight;
    property Visible   : Boolean    read FVisible  write SetVisible  default True;
    property Width     : Integer    read FWidth    write SetWidth    default 250;
    property Height    : Integer    read FHeight   write SetHeight   default 200;
    property TimeStamp : Boolean    read FStamp    write FStamp      default False;
  end;

implementation

constructor TDebugBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPosition := poTopRight;
  FVisible  := False;
  FWidth    := 250;
  FHeight   := 200;
  FIndent   := 0;
  FCaption  := 'Debug mode';
  DebugForm := TForm.Create(Application);
  with DebugForm do
  begin
    Visible     := FVisible;
    Caption     := FCaption;
    FormStyle   := fsStayOnTop;
    BorderStyle := bsSizeToolWin;
  end;
  DebugList := TListBox.Create(DebugForm);
  with DebugList do
  begin
    Parent    := DebugForm;
    Align     := alClient;
    Sorted    := False;
    Font.Name := 'Small Fonts';
    Font.Size := 7;
  end;
end;

procedure TDebugBox.SetPosition(Value: TPositions);
begin
  FPosition := Value;
  with DebugForm do
    case Value of
      poTopleft     : SetBounds(0, 0, Width, Height);
      poBottomLeft  : SetBounds(0, Screen.Height - Height, Width, Height);
      poBottomRight : SetBounds(Screen.Width - Width, Screen.Height - Height, Width, Height);
      poTopRight    : SetBounds(Screen.Width - Width, 0, Width, Height);
    end;
end;

procedure TDebugBox.SetVisible (Value: Boolean);
begin
  FVisible := Value;
  if(csDesigning in ComponentState) then Exit;
  DebugForm.Hide;
  if Value then
  begin
    Width  := Self.Width;
    Height := Self.Height;
    SetPosition(FPosition);
    DebugForm.Show;
  end;
end;

procedure TDebugBox.SetWidth(Value: Integer);
begin
  FWidth := Value;
  DebugForm.Width := FWidth;
  SetPosition(FPosition);
end;

procedure TDebugBox.SetHeight  (Value: integer);
begin
  FHeight := Value;
  DebugForm.Height := FHeight;
  SetPosition(FPosition);
end;

procedure TDebugBox.SetCaption (Value: String);
begin
  FCaption := Value;
  DebugForm.Caption := FCaption;
end;

procedure TDebugBox.Clear;
begin
  DebugList.Items.Clear;
  DebugList.ItemIndex := DebugList.Items.Count - 1;
end;

procedure TDebugBox.Add(Value: String);
const Space            = '  ';
var
  Text, Chaine : string;
  I            : integer;

  procedure AddChaine;

    function theSpace : string;
    var J : integer;
    begin
      Result := '';
      for J := 1 to FIndent do
        result := result + Space;
    end;

  begin
    if (Chaine <> '') and (Chaine <> #0) then
    begin
      if TimeStamp then
        DebugList.Items.Add(TimeToStr(Time) + ' - ' + theSpace + Chaine)
      else
        DebugList.Items.Add(theSpace + Chaine);
    end;
    DebugList.ItemIndex := DebugList.Items.Count - 1;
    Chaine := '';
  end;

begin
  Text := Value + #0;
  chaine := '';
  I := 1;
  while i <= length(Text) do
  begin
    case UpCase(Text[I]) of
      '>' : begin
              AddChaine;
              Inc(FIndent);
            end;
      '<' : begin
              AddChaine;
              Dec(FIndent);
            end;
       #0 : AddChaine;
      else Chaine := Chaine + Text[I];
    end;
    inc(I);
  end;
end;

end.
