(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit FormsUtils;

interface

uses Forms, Graphics;

procedure CenterForm(Form: TForm);

{------------------------------ Gestion des fonts -----------------------------}
function FontToStr (Value: TFont) : string;
procedure StrToFont (S : string; FFont : TFont);

{------------------------------ Gestion des forms -----------------------------}
function Form2Str (Form : TForm) : string;
procedure Str2Form (S : string; Form : TForm);

resourcestring
  Separateur = '||';

implementation

uses SysUtils;

procedure CenterForm(Form: TForm);
begin
  if not Assigned(Application.MainForm) then Exit;
  Form.Left := ((Application.MainForm.Width  - Form.Width ) DIV 2) + Application.MainForm.Left;
  Form.Top  := ((Application.MainForm.Height - Form.Height) DIV 2) + Application.MainForm.Top;
  if Form.Left < 0 then Form.Left := 0;
  if Form.Left + Form.Width > Screen.Width then Form.Left := Screen.Width - Form.Width;
  if Form.Top < 0 then Form.Top := 0;
  if Form.Top + Form.Height > Screen.Height then Form.Top := Screen.Height - Form.Height;
end;

function FontToStr (Value: TFont) : string;
var Str : string;
begin
  Str := '';
  Str := Value.NAME + Separateur;
  Str := Str + ColorToString (Value.Color) + Separateur;
  Str := Str + IntToStr (Value.Size) + Separateur;
  if fsBold      in Value.Style then Str := Str + 'b';
  if fsItalic    in Value.Style then Str := Str + 'i';
  if fsUnderline in Value.Style then Str := Str + 'u';
  if fsStrikeOut in Value.Style then Str := Str + 's';
  Result := Str;
end;

procedure StrToFont (S : string; FFont : TFont);
var Str : string;
 I   : byte;
begin
  Str := S;

  {read font name}

  I := Pos (Separateur, Str);
  if i = 0 then exit;
  FFont.NAME := Copy (Str, 1, i - 1);
  Delete(Str, 1, i + length (Separateur) - 1);

  {read font color}

  i := Pos (Separateur, Str);
  if i = 0 then exit;
  try
    FFont.Color := StringtoColor (Copy (Str, 1, i - 1));
  except
    FFont.Free;
    exit;
  end;
  Delete (Str, 1, i + length (separateur) - 1);

  {read font size}

  i := Pos (Separateur, Str);
  if i = 0 then exit;
  try
    FFont.Size:= StrToInt (Copy (Str, 1, i - 1));
  except
    FFont.Free;
    exit;
  end;
  Delete (Str, 1, i + length (Separateur) - 1);

  {read font style}

  FFont.Style:=[];
  while byte (Length (Str)) > 0 do
  begin
    case Str [1] of
      'b': FFont.Style := FFont.Style + [fsbold];
      'i': FFont.Style := FFont.Style + [fsitalic];
      'u': FFont.Style := FFont.Style + [fsUnderline];
      's': FFont.Style := FFont.Style + [fsStrikeout];
    end;
    Delete(Str , 1, 1);
  end;
end;

function Form2Str (Form : TForm) : string;
begin
  if Form.WindowState = wsMaximized
    then result := '1'
    else result := '0';
  Form.WindowState := wsNormal;
  Result := IntToStr (Form.Left) + ',' + IntToStr (Form.Top) + ',' +
            IntToStr (Form.Height) + ',' + IntToStr (Form.Width) + ',' + Result;
end;

procedure Str2Form (S : string; Form : TForm);
begin
  if S = '' then Exit;
  Form.Left := StrToInt (Copy (S, 1, pos (',', S) - 1));
  Delete (S, 1, Pos (',', S));
  Form.Top := StrToInt (Copy (S, 1, pos (',', S) - 1));
  Delete (S, 1, Pos (',', S));
  Form.Height := StrToInt (Copy (S, 1, pos (',', S) - 1));
  Delete (S, 1, Pos (',', S));
  Form.Width := StrToInt (Copy (S, 1, pos (',', S) - 1));
  Delete (S, 1, Pos (',', S));
  if StrToInt (S) = 1
    then Form.WindowState := wsMaximized
    else Form.WindowState := wsNormal;
end;

end.
