(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit tShellAPI;

interface

{$H+}

uses Graphics, Classes;

(* Fonctions dfinie dans Windows 'Shell32' *)

type
  TFileOperation     = (foCopy, foDelete, foMove, foRename);
  TFileOperFlag      = (fofAllowUndo, fofConfirmMouse, fofFilesOnly,
                        fofMultiDestFiles, fofNoConfirmation, fofNoConfirmMkDir,
                        fofRenameOnCollision, fofSilent, fofSimpleProgress,
                        fofWantMappingHandle);
  TFileOperFlags     = set of TFileOperFlag;
  TShellFileOpStruct = record
                         wFunc                 : TFileOperation;
                         pFrom, pTo            : string;
                         fFlags                : TFileOperFlags;
                         fAnyOperationsAborted : Boolean;
                         lpszProgressTitle     : string;
                       end;

function ShellFileOperation (Struct : TShellFileOpStruct) : boolean;
procedure ShellEmptyRecycleBin(GUI, Sound : boolean); overload;
procedure ShellEmptyRecycleBin; overload;

(* Fonction complmentaire *)

function CopyFileByShell(const FileName, DestName: PChar;
                         Confirmation, AllowUndo, MoveFile: Boolean) : boolean;

function ShellIconExtract(const FileName: string; Id: Integer): TIcon;

implementation

uses Windows, ShellApi, SysUtils;

Const
   SHERB_NOCONFIRMATION = $00000001;
   SHERB_NOPROGRESSUI   = $00000002;
   SHERB_NOSOUND        = $00000004;

Type
  TSHEmptyRecycleBin = function(Wnd : HWND; pszRootPath : PChar;
                                dwFlags : DWORD): HRESULT; stdcall ;

procedure ShellEmptyRecycleBin;
begin
  ShellEmptyRecycleBin(false, false);
end;

procedure ShellEmptyRecycleBin(GUI, Sound : boolean);
Var
  SHEmptyRecycleBin : TSHEmptyRecycleBin;
  LibHandle         : THandle;
  fl                : DWord;
begin  { EmptyRecycleBin }
  LibHandle := LoadLibrary('Shell32.dll') ;
  if LibHandle = 0 then
  begin
    MessageBox(0, 'Failed to load Shell32.dll.', 'Error', MB_OK or MB_ICONERROR);
    Exit;
  end;
  try
    @SHEmptyRecycleBin := GetProcAddress(LibHandle, 'SHEmptyRecycleBinA');
    fl := SHERB_NOCONFIRMATION;
    if not GUI then fl := fl or SHERB_NOPROGRESSUI;
    if not Sound then fl := fl or SHERB_NOSOUND;
    if @SHEmptyRecycleBin <> nil
      then SHEmptyRecycleBin(MainInstance, nil, fl);
  finally
    FreeLibrary(LibHandle);
    @SHEmptyRecycleBin := nil ;
  end;
end;  { EmptyRecycleBin }

const
  OperTypes   : array [TFileOperation]
                of Integer = (FO_COPY, FO_DELETE, FO_MOVE, FO_RENAME);
  OperOptions : array [TFileOperFlag]
                of Integer = (FOF_ALLOWUNDO, FOF_CONFIRMMOUSE, FOF_FILESONLY,
                              FOF_MULTIDESTFILES, FOF_NOCONFIRMATION,
                              FOF_NOCONFIRMMKDIR, FOF_RENAMEONCOLLISION,
                              FOF_SILENT, FOF_SIMPLEPROGRESS, FOF_WANTMAPPINGHANDLE);

function ShellFileOperation (Struct : TShellFileOpStruct) : boolean;
var
  OpStruct : TSHFileOpStruct;
  Flag     : TFileOperFlag;

  function AllocFileStr(const S: string): PChar;
  var
    P: PChar;
  begin
    Result := nil;
    if S <> '' then begin
      Result := StrCopy(StrAlloc(Length(S) + 2), PChar(S));
      P := Result;
      while P^ <> #0 do begin
        if (P^ = ';') or (P^ = '|') then P^ := #0;
        Inc(P);
      end;
      Inc(P);
      P^ := #0;
    end;
  end;

begin
  FillChar(OpStruct, SizeOf(OpStruct), 0);
  with OpStruct do
  begin
    Wnd    := MainInstance;
    wFunc  := OperTypes [Struct.wFunc];
    pFrom  := AllocFileStr(Struct.pFrom);
    pTo    := AllocFileStr(Struct.pTo);
    fFlags := 0;
    for Flag := Low(Flag) to High(Flag)
      do if Flag in Struct.fFlags
        then fFlags := fFlags or OperOptions [Flag];
    lpszProgressTitle := PChar(Struct.lpszProgressTitle);
    result                       := Boolean (SHFileOperation(OpStruct));
    Struct.fAnyOperationsAborted := fAnyOperationsAborted;
  end;
end;

function ShellIconExtract(const FileName: string; Id: Integer): TIcon;
var
  FileNm:string;
  S: array[0..255] of char;
  IconHandle: HIcon;
  Index: Word;
  i:integer;
begin
  Result := TIcon.Create;
  try
    FileNm := FileName;
    for i := 1 to length(FileNm) do
      if FileNm[I] = '"' then
        Delete(FileNm, I, 1);
    StrPLCopy(S, FileNm, SizeOf(S) - 1);
    IconHandle := ExtractIcon(hInstance, S, Id);
    if IconHandle < 2 then begin
      Index := Id;
      IconHandle := ExtractAssociatedIcon(hInstance, S, Index);
    end;
    if IconHandle < 2 then begin
      if IconHandle = 1 then
        raise EResNotFound.Create('No Icon.')
      else begin
        Result.Free;
        Result := nil;
      end;
    end else Result.Handle := IconHandle;
  except
    Result.Free;
    raise;
  end;
end;

function CopyFileByShell(const FileName, DestName: PChar;
                         Confirmation, AllowUndo, MoveFile: Boolean) : boolean;
var SHFileSt : TShellFileOpStruct;
begin
  with ShFileSt do
  begin
    if MoveFile
      then wFunc := foMove
      else wFunc := foCopy;
    if DestName = nil
      then wFunc := foDelete;
    pFrom  := FileName;
    pTo    := DestName;
    fFlags := [fofSilent];
    if not Confirmation then fFlags := fFlags + [fofNoConfirmation];
    if     AllowUndo    then fFlags := fFlags + [fofAllowUndo];
    result := ShellFileOperation (SHFileSt);
    result := result and fAnyOperationsAborted;
  end;
end;

end.
